<?php

namespace Drupal\hijri_format\Plugin\Field\FieldFormatter;

use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\hijri_format\HijriFormatManager;

/**
 * Plugin implementation of the 'hijri_field_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "hijri_field_formatter",
 *   label = @Translation("Hijri"),
 *   field_types = {
 *     "datetime",
 *     "daterange",
 *     "created",
 *     "changed"
 *   }
 * )
 */
class HijriFieldFormatter extends FormatterBase implements ContainerFactoryPluginInterface {
  /**
   * Indian Numbers.
   *
   * @var bool
   */
  private $indian;
  /**
   * Hijri manager.
   *
   * @var \Drupal\hijri_format\HijriFormatManager
   */
  protected $hijriFormartManager;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, HijriFormatManager $hijriManager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->hijriFormartManager = $hijriManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('hijri_format.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      // Implement default settings.
      'format' => 'd-M-Y',
      'is_indian' => 0,
      'seperator' => 'To',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();

    $form = $this->hijriFormartManager->buildForm($settings);

    if ($this->fieldDefinition->getType() == "daterange") {
      $form['seperator'] = [
        '#title' => 'Seperator',
        '#type' => 'textfield',
        '#default_value' => $this->getSetting("seperator"),
      ];
    }
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $settings = $this->getSettings();

    $is_indian = ($settings['is_indian'] == 1) ? t("yes") : t("No");

    $summary[] = t('Format:') . ' ' . $settings['format'];
    $summary[] = t('Indian Numbers:') . ' ' . $is_indian;

    if ($this->fieldDefinition->getType() == "daterange") {
      $summary[] = t('Seperator:') . ' ' . $this->getSetting('seperator');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $date = [];
    foreach ($items as $delta => $item) {
      $format = $this->getSetting('format');
      $is_indian = ($this->getSetting('is_indian')) ? TRUE : FALSE;
      $seperator = $this->getSetting('seperator');
      $class = get_class($item);
      switch ($class) {
        case 'Drupal\datetime_range\Plugin\Field\FieldType\DateRangeItem':
          $date[] = $this->hijriFormartManager->convertToHijri(strtotime($item->getValue()['value']), $format, $is_indian);
          $date[] = $this->hijriFormartManager->convertToHijri(strtotime($item->getValue()['end_value']), $format, $is_indian);
          break;

        case "Drupal\datetime\Plugin\Field\FieldType\DateTimeItem":
          $value = strtotime($item->getValue()['value']);
        default:
          $value = isset($value) ? $value : $item->getValue()['value'];
          $date[] = $this->hijriFormartManager->convertToHijri($value, $format, $is_indian);
      }
      $elements[$delta] = [
        '#theme' => 'hijri',
        '#date' => $date,
        '#seperator' => $seperator,
      ];
    }
    return $elements;
  }

}
